/* indybot - Copyright (C) 2004 bakunin - <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <indybot.h>

void
check (struct channel *chan)
{
  struct file *fl;

  fl = chan->files;

  while (fl)
    {
      if (debug)
	fprintf (stderr, "Downloading file '%s' for '%s' channel...\n",
		 fl->file, chan->channel);

      parse (fl->file, chan, fl);

      fl = fl->next;
    }
}

void
parse_backslash_story (nxml_t * doc, nxml_data_t * cur, struct channel *chan,
		       struct file *fl)
{
  char *title, *author, *url;
  static char tmp[SIZE_BUFFER];
  int len;

  title = url = author = NULL;

  cur = cur->children;

  while (cur)
    {
      if (cur->type == NXML_TYPE_ELEMENT)
	{
	  if (!strcmp (cur->value, "title") && !title)
	    nxml_get_string (cur, &title);

	  else if (!strcmp (cur->value, "url") && !url)
	    nxml_get_string (cur, &url);

	  else if (!strcmp (cur->value, "author") && !author)
	    nxml_get_string (cur, &author);
	}

      cur = cur->next;
    }

  if (!title || !url || !author)
    {
      if (title)
	free (title);

      if (url)
	free (url);

      if (author)
	free (author);

      return;
    }

  if (fl->current_id < get_story_id (url))
    fl->current_id = get_story_id (url);

  if (!fl->last_id)
    {
      if (debug)
	fprintf (stderr, "Cache empty: add ID %d.\n", get_story_id (url));

      fl->last_id = get_story_id (url);
      return;
    }

  else if (fl->last_id >= get_story_id (url))
    {
      if (debug)
	fprintf (stderr, "Check ID %d < of the ID cache %d\n",
		 get_story_id (url), fl->last_id);
    }

  else
    {
      char *n_url, *n_title, *n_author;

#ifdef HAVE_LIBRECODE
      n_url = recode (url);
      n_title = recode (trim (title));
      n_author = recode (trim (author));
#else
      n_url = url;
      n_title = trim (title);
      n_author = trim (author);
#endif

      len =
	snprintf (tmp, SIZE_BUFFER, "PRIVMSG %s :%s - by %s - %s\n",
		  chan->channel, n_title, n_author, n_url);

#ifdef HAVE_LIBRECODE
      free (n_url);
      free (n_title);
      free (n_author);
#endif

      if (len)
	irc_write (tmp, len);
    }

  free (title);
  free (url);
  free (author);

  return;
}

void
parse (char *file, struct channel *chan, struct file *fl)
{

  nxml_t *doc;
  nxml_data_t *cur;

  if (nxml_new (&doc) != NXML_OK)
    {
      fprintf (stderr, "Error: nxml init.\n");
      return;
    }

  if (nxml_parse_url (doc, file) != NXML_OK)
    {
      fprintf (stderr, "Error: nxml parse url.\n");
      nxml_free (doc);
      return;
    }

  if (nxml_root_element (doc, &cur) != NXML_OK || !cur)
    {
      fprintf (stderr, "Error: no info in xml file.\n");
      nxml_free (doc);
      return;
    }

  if (!strcmp (cur->value, "backslash"))
    parse_backslash (doc, cur, chan, fl);
  else if (!strcmp (cur->value, "RDF"))
    parse_rdf (doc, cur, chan, fl);
  else
    fprintf (stderr, "Error: parse xml file.\n");

  nxml_free (doc);
}

void
parse_backslash (nxml_t * doc, nxml_data_t * cur, struct channel *chan,
		 struct file *fl)
{
  cur = cur->children;

  if (debug)
    fprintf (stderr, "Check backslash xml structs...\n");

  fl->current_id = 0;

  while (cur != NULL)
    {
      if (cur->type == NXML_TYPE_ELEMENT && !strcmp (cur->value, "story"))
	parse_backslash_story (doc, cur, chan, fl);

      cur = cur->next;
    }

  if (fl->current_id > fl->last_id)
    fl->last_id = fl->current_id;

  if (debug)
    fprintf (stderr, "Parsing OK\n");
}

void
parse_rdf (nxml_t * doc, nxml_data_t * cur, struct channel *chan,
	   struct file *fl)
{
  cur = cur->children;

  if (debug)
    fprintf (stderr, "Check backslash xml structs...\n");

  fl->current_id = 0;

  while (cur != NULL)
    {
      if (cur->type == NXML_TYPE_ELEMENT && !strcmp (cur->value, "item"))
	parse_rdf_item (doc, cur, chan, fl);

      cur = cur->next;
    }

  if (fl->current_id > fl->last_id)
    fl->last_id = fl->current_id;

  if (debug)
    fprintf (stderr, "Parsing OK\n");
}

void
parse_rdf_item (nxml_t * doc, nxml_data_t * cur, struct channel *chan,
		struct file *fl)
{
  char *title, *description, *url;
  static char tmp[SIZE_BUFFER];
  int len;

  title = url = description = NULL;

  cur = cur->children;

  while (cur)
    {
      if (cur->type == NXML_TYPE_ELEMENT)
	{
	  if (!strcmp (cur->value, "title") && !title)
	    nxml_get_string (cur, &title);

	  else if (!strcmp (cur->value, "link") && !url)
	    nxml_get_string (cur, &url);

	  else if (!strcmp (cur->value, "description") && !description)
	    nxml_get_string (cur, &description);
	}

      cur = cur->next;
    }

  if (!title || !url || !description)
    {
      if (title)
	free (title);

      if (url)
	free (url);

      if (description)
	free (description);

      return;
    }

  if (fl->current_id < get_rdf_id (url))
    fl->current_id = get_rdf_id (url);

  if (!fl->last_id)
    {
      if (debug)
	fprintf (stderr, "Cache empty: add ID %d.\n", get_rdf_id (url));

      fl->last_id = get_rdf_id (url);
      return;
    }

  else if (fl->last_id >= get_rdf_id (url))
    {
      if (debug)
	fprintf (stderr, "Check ID %d < of the ID cache %d\n",
		 get_rdf_id (url), fl->last_id);
    }

  else
    {
      char *n_url, *n_title, *n_description;

#ifdef HAVE_LIBRECODE
      n_url = recode (url);
      n_title = recode (trim (title));
      n_description = recode (trim (description));
#else
      n_url = url;
      n_title = trim (title);
      n_description = trim (description);
#endif

      len =
	snprintf (tmp, SIZE_BUFFER, "PRIVMSG %s :%s - %s - %s\n",
		  chan->channel, n_title, n_description, n_url);

#ifdef HAVE_LIBRECODE
      free (n_url);
      free (n_title);
      free (n_description);
#endif

      if (len)
	irc_write (tmp, len);
    }

  free (title);
  free (url);
  free (description);

  return;
}

/* EOF */
